# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# N.B. Be careful what is imported in this file; it is included in setup.py
#      so dependencies are not necessarily installed.

__version__ = '0.3.0'
